def cx(control, target, vector, n):
    num = 2**(n - 2)
    pos1 = ['0'] * n
    for i in range(num):
        k = bin(i)[2:]
        m = 0
        for j in range(n):
            if j == control:
                pos1[j] = '1'
            elif j == target:
                pos1[j] = '0'
            else:
                pos1[j] = k[m]
                m += 1
        pos2 = pos1.copy()
        pos2[target] = '1'
        pos1.reverse()
        pos2.reverse()
        x = int(''.join(pos1), 2)
        y = int(''.join(pos2), 2)
        vector[x], vector[y] = vector[y], vector[x]

    return vector



from ivav.index_view import IndexView
import pytest

PARAMETERS = [
    [3, [3, 1, 4, 1, 5, 9, 2, 6], [1, 3, 1, 4, 9, 5, 6, 2], 0],
    [3, [3, 1, 4, 1, 5, 9, 2, 6], [4, 1, 3, 1, 2, 6, 5, 9], 1],
    [3, [3, 1, 4, 1, 5, 9, 2, 6], [5, 9, 2, 6, 3, 1, 4, 1], 2],
]


@pytest.mark.parametrize("n, state_vector, expected_state_vector, index",
                         PARAMETERS)
def test_not_gate(n, state_vector, expected_state_vector, index):
    index_view = IndexView(n, state_vector)
    index_view.x(index)
    assert index_view.state_vector == expected_state_vector


def hadamard_gate(index, vector, size):
    start = 0
    end = 2**index
    dist = end - start

    while start < size:
        last = start + dist
        a = vector[start]
        b = vector[last]
        vector[start] = a + b
        vector[last] = a - b
        start += 1
        if start == end:
            start = end + dist
            end = start + dist

    return vector



from setuptools import setup

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(name="ivav",
      version="0.0.2",
      description="A quantum computer simulator",
      py_modules=["index_view"],
      package_dir={"": "ivav"},
      classifiers=[
          "Programming Language :: Python :: 3",
          "Programming Language :: Python :: 3.6",
          "License :: OSI Approved :: MIT License",
          "Operating System :: OS Independent",
      ],
      extras_require={
          "dev": ["pytest>=7.0.1", "check-manisfest>=0.47", "yapf>=0.32.0"]
      },
      long_description=long_description,
      long_description_content_type="text/markdown",
      url="https://github.com/Gopal-Dahale/ivav",
      author="Gopal Ramesh Dahale",
      author_email="dahalegopal27@gmail.com")



from ivav.gates.single_qubit.pauli import x, y, z
from ivav.gates.single_qubit import h
from ivav.gates.multi_qubit.control import ccx, cx


class IndexView:

    def __init__(self, n, state_vector):
        self.n = n
        self.state_vector = state_vector
        self.len = len(state_vector)
        assert self.len == 2**n, "The state vector must be of length 2**n"

    def x(self, index):
        assert index < self.n, "Index must be less than n"
        self.vector = x.pauli_x(index, self.state_vector, self.len)

    # Yet to be implemented
    def y(self, index):
        self.vector = y.pauli_y(index, self.state_vector)

    # Yet to be implemented
    def z(self, index):
        self.vector = z.pauli_z(index, self.state_vector)

    def h(self, index):
        self.vector = h.hadamard_gate(index, self.state_vector, self.len)

    def cx(self, control, target):
        assert control < self.n, "Control index must be less than n"
        assert target < self.n, "Target index must be less than n"
        self.vector = cx.cx(control, target, self.state_vector, self.n)

    def ccx(self, control1, control2, target):
        assert control1 < self.n, "Control1 index must be less than n"
        assert control2 < self.n, "Control2 index must be less than n"
        assert target < self.n, "Target index must be less than n"
        self.vector = ccx.ccx(control1,control2,target, self.state_vector,self.n)